from configparser import ConfigParser
from django.core.management.base import BaseCommand

from ...models import PaymentSystem


class Command(BaseCommand):
    help = "Upload file payment_systems.ini to db"

    def add_arguments(self, parser):
        parser.add_argument(
            "-f",
            "--file",
            required=True,
            help="path to payment_systems.ini"
        )

    def handle(self, *args, **options):
        file_path = options.get("file")
        config = ConfigParser()
        config.read(file_path)
        i = 0
        for section in config.sections():
            values = {}
            values["group"] = config[section]["group"]
            values["title"] = config[section]["title"]
            values["account_id"] = config[section]["accountId"]
            values["unit_id"] = config[section]["unitId"]
            values["create_invoice"] = config[section].getboolean("createInvoice")
            values["name"] = section.replace("monetasdk_paysys_", "")
            
            PaymentSystem.objects.create(**values)
            i += 1
            
        self.stdout.write(self.style.SUCCESS(f"uploaded {i} payment systems"))





import datetime
from typing import List, OrderedDict
from pydash import py_

import zeep
from zeep.wsse.username import UsernameToken

import logging


class SoapClient:
    def __init__(self, url: str, login: str, password: str):
        token = UsernameToken(login, password)
        self.client = zeep.Client(url, wsse=token)

    @staticmethod
    def debug_mode():
        logging.basicConfig(level=logging.DEBUG)
        logging.getLogger("suds.client").setLevel(logging.DEBUG)

    @staticmethod
    def _convert_to_list_of_dicts(operations: List[OrderedDict]):
        results = []
        for item in operations:
            elm = dict()
            elm["id"] = item.get('id')
            for d in item.get('attribute'):
                elm[d['key']] = d['value'] if d['key'] != 'sourceamount' else int(d['value'])
            results.append(elm)
        return results

    def _get_operation_list(
        self, date_from: datetime.datetime, date_to: datetime.datetime
    ) -> List[OrderedDict]:
        ftr = {"dateFrom": date_from, "dateTo": date_to}
        response = self.client.service.FindOperationsList(filter=ftr)
        result = zeep.helpers.serialize_object(response)
        return result.get("operation", [])

    def get_operations(self, date_from: datetime.datetime, date_to: datetime.datetime) -> List[dict]:
        result = self._get_operation_list(date_from, date_to)
        return self._convert_to_list_of_dicts(result)

    def get_sum_per_period(
        self, date_from: datetime.datetime, date_to: datetime.datetime
    ) -> int:
        operations = self._get_operation_list(date_from, date_to)
        return (
            py_(operations)
            .map("attribute")
            .flatten()
            .reject(lambda elm: py_.get(elm, ["key"]) != "sourceamount")
            .map_("value")
            .map(int)
            .sum()
            .value()
        )

    def get_operation_sum(self, operation_id: int) ->int:
        # result = self.client.service.
        pass




# Generated by Django 3.2.3 on 2021-05-30 14:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('moneta', '0002_load_payment_systems'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentsystem',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('merchant_id', models.CharField(help_text='MNT_ID', max_length=30)),
                ('transaction_id', models.CharField(max_length=255, unique=True)),
                ('operation_id', models.CharField(blank=True, null=True, help_text='MNT_OPERATION_ID', max_length=30)),
                ('amount', models.DecimalField(decimal_places=2, help_text='MNT_AMOUNT', max_digits=11)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('currency', models.CharField(help_text='MNT_CURRENCY_CODE', max_length=3)),
                ('test_mode', models.BooleanField(help_text='MNT_TEST_MODE')),
                ('signature', models.CharField(blank=True, help_text='MNT_SIGNATURE', max_length=250)),
                ('payment_system_unit_id', models.CharField(help_text='paymentSystem.unitId', max_length=10, blank=True)),
                ('corraccount', models.CharField(blank=True, null=True, help_text='MNT_CORRACCOUNT', max_length=20)),
                ('subscriber_id', models.CharField(blank=True, help_text='MNT_SUBSCRIBER_ID', max_length=200)),
                ('status', models.CharField(choices=[('CHECK', 'CHECK'), ('PAID', 'PAID')], max_length=6)),
                ('payment_system', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='moneta.paymentsystem')),
            ],
            options={
                'db_table': 'mnt_invoices',
            },
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-10 20:05

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentSystem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.CharField(choices=[('electronic', 'electronic'), ('bank', 'bank')], max_length=10)),
                ('title', models.CharField(max_length=50)),
                ('name', models.CharField(max_length=30)),
                ('account_id', models.PositiveIntegerField()),
                ('unit_id', models.CharField(max_length=100)),
                ('create_invoice', models.BooleanField()),
            ],
            options={
                'db_table': 'mnt_payment_systems',
            },
        ),
    ]



from setuptools import (
    setup,
)

extras_require = {
    "test": [
        "pytest-django",
        "pytest",
        "pytest-dotenv",
    ],
    "dev": [
        "ipython",
    ],
}

extras_require["dev"] = extras_require["dev"] + extras_require["test"]


setup(
    name="django-payanyway",
    url="https://github.com/r-pletnev/django-moneta",
    license="GPL",
    description="Moneta(PayAnyWay)",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Roman Pletnev",
    author_email="ge52@mail.ru",
    install_requires=["django>=2", "pydantic", "zeep", "pydash"],
    python_require=">=3.7",
    extras_require=extras_require,
    packages=["moneta"],
    include_package_data=True,
    zip_safe=False,
)



from django.contrib import admin

from .models import PaymentSystem, Invoice


@admin.register(PaymentSystem)
class PaymentSystemAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "title", "group")
    list_filter = ("group",)


@admin.register(Invoice)
class InvoiceAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "transaction_id",
        "created",
        "status",
        "amount",
        "currency",
        "test_mode",
    )



from django import forms

from .models import PaymentSystem


class PaymentForm(forms.Form):
    transaction_id = forms.IntegerField(min_value=1)
    amount = forms.FloatField(min_value=1)
    payment_system = forms.ModelChoiceField(queryset=PaymentSystem.objects.all())
